
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v5.031.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "LWP tracing and debugging",
  "dist_author" => [
    "Olaf Alders <olaf\@wundercounter.com>"
  ],
  "dist_name" => "LWP-ConsoleLogger",
  "dist_version" => "0.000018",
  "license" => "artistic_2",
  "module_name" => "LWP::ConsoleLogger",
  "recommends" => {
    "HTML::FormatText::Lynx" => 23
  },
  "recursive_test_files" => 1,
  "requires" => {
    "Data::Printer" => 0,
    "DateTime" => 0,
    "HTML::Restrict" => 0,
    "HTTP::Body" => 0,
    "HTTP::CookieMonster" => 0,
    "JSON::MaybeXS" => 0,
    "Log::Dispatch" => 0,
    "Module::Load::Conditional" => 0,
    "Moo" => 0,
    "MooX::StrictConstructor" => 0,
    "Parse::MIME" => 0,
    "String::Trim" => 0,
    "Sub::Exporter" => 0,
    "Term::Size::Any" => 0,
    "Text::SimpleTable::AutoWidth" => "0.09",
    "Try::Tiny" => 0,
    "Type::Tiny" => 0,
    "Types::Common::Numeric" => 0,
    "Types::Standard" => 0,
    "URI::Query" => 0,
    "URI::QueryParam" => 0,
    "XML::Simple" => 0,
    "perl" => "5.006",
    "strict" => 0,
    "warnings" => 0
  },
  "script_files" => [],
  "test_requires" => {
    "HTML::FormatText::WithLinks" => 0,
    "HTTP::Request" => 0,
    "LWP::UserAgent" => 0,
    "Log::Dispatch::Array" => 0,
    "Path::Tiny" => 0,
    "Plack::Handler::HTTP::Server::Simple" => "0.016",
    "Plack::Test" => 0,
    "Plack::Test::Agent" => 0,
    "Test::FailWarnings" => 0,
    "Test::Fatal" => 0,
    "Test::Most" => 0,
    "URI::file" => 0,
    "WWW::Mechanize" => 0
  }
);


my %fallback_build_requires = (
  "HTML::FormatText::WithLinks" => 0,
  "HTTP::Request" => 0,
  "LWP::UserAgent" => 0,
  "Log::Dispatch::Array" => 0,
  "Module::Build" => "0.28",
  "Path::Tiny" => 0,
  "Plack::Handler::HTTP::Server::Simple" => "0.016",
  "Plack::Test" => 0,
  "Plack::Test::Agent" => 0,
  "Test::FailWarnings" => 0,
  "Test::Fatal" => 0,
  "Test::Most" => 0,
  "URI::file" => 0,
  "WWW::Mechanize" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);

$build->create_build_script;
